import { useState, useCallback } from 'react';

var isArray = Array.isArray;
var isFunction = function isFunction(value) {
  return value instanceof Function;
};
var isMap = function isMap(value) {
  return value instanceof Map;
};
var isPlainObject = function isPlainObject(value) {
  return typeof value === "object" && (value == null ? void 0 : value.constructor) === Object;
};
var isSet = function isSet(value) {
  return value instanceof Set;
};

var getReturnValue = function getReturnValue(value) {
  if (isFunction(value)) {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    return value.apply(void 0, args);
  } else {
    return value;
  }
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var mergeArrays = function mergeArrays(a, b) {
  return [].concat(a, b);
};

var mergePlainObjects = function mergePlainObjects(a, b) {
  return _extends({}, a, b);
};

var mergeMaps = function mergeMaps(a, b) {
  return new Map([].concat(a, b));
};

var mergeSets = function mergeSets(a, b) {
  return new Set([].concat(a, b));
};

var merge = function merge(a, b) {
  if (isArray(a) && isArray(b)) {
    return mergeArrays(a, b);
  } else if (isPlainObject(a) && isPlainObject(b)) {
    return mergePlainObjects(a, b);
  } else if (isMap(a) && isMap(b)) {
    return mergeMaps(a, b);
  } else if (isSet(a) && isSet(b)) {
    return mergeSets(a, b);
  } else {
    return b;
  }
};

var defaultOptions = {
  merge: true
};
var useMergeState = function useMergeState(initialState) {
  var _useState = useState(initialState),
      state = _useState[0],
      setState = _useState[1];

  var setMergeState = useCallback(function (value, options) {
    var _options;

    if (options === void 0) {
      options = defaultOptions;
    }

    var _defaultOptions = _extends({}, defaultOptions, (_options = options) != null ? _options : {}),
        shouldMerge = _defaultOptions.merge;

    if (shouldMerge) {
      setState(function (previousState) {
        return merge(previousState, getReturnValue(value, previousState));
      });
    } else {
      setState(value);
    }
  }, []);
  return [state, setMergeState];
};

export { merge, useMergeState };
//# sourceMappingURL=use-merge-state.esm.js.map
